<?php

/**
 * @file robotagger.admin.inc.
 */

function robotagger_settings_page() {
  $output = '';
  $output .= drupal_get_form('robotagger_settings_form');
  return $output;
}


function robotagger_settings_form($form_state) {
  _robotagger_add_js();
  $form = array();
  $form['robotagger_global_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#description' => t('Please note. You need to assign the content types for each vocabularies.'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );
  $form['robotagger_global_wrapper']['robotagger_global_sendrequest'] = array(
    '#type' => 'select',
    '#title' => t('Send Request'),
    '#options' => array('disabled' => t('disabled'), 'auto' => t('automatic'), 'manual' => t('manual')),
    '#default_value' => variable_get('robotagger_global_sendrequest', 'auto'),
    '#description' => t('automatic - after submitting a node, request the RoboTagger service'),
  );
  $form['robotagger_global_wrapper']['robotagger_global_confirmtags'] = array(
    '#type' => 'select',
    '#title' => t('Confirm returned tags'),
    '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
    '#default_value' => variable_get('robotagger_global_confirmtags', 'auto'),
    '#description' => t('manual - Before the tags are saved, you can confirm or edit them. automatic - the module save the tags immediately'),
  );
  $nodetypes = node_get_types('names');
  $form['robotagger_global_wrapper']['robotagger_global_nodetypes'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Supported Nodetypes'),
    '#options' => $nodetypes,
    '#default_value' => variable_get('robotagger_global_nodetypes', array()),
    '#description' => t('Choose the node types for RoboTagger'),
  );
  $languages = _robotagger_get_all_supported_languages();
  $options = array();
  foreach ($languages as $langcode => $langname) {
    $options = _robotagger_get_vacabularies($langcode, 'db');
    if (empty($options)) {
      continue;
    }
    $values = variable_get('robotagger_global_vocs_'.$langcode, array());
    $form['robotagger_global_wrapper']['robotagger_global_vocs_'.$langcode] = array(
      '#type' => 'checkboxes',
      '#title' => t('Vocabularies for the language @language', array('@language' => $langname)),
      '#options' => $options,
      '#default_value' => $values,
    );
    $text = array_filter($values) ? t('Deselect all') : t('Select all');
    $form['robotagger_global_wrapper']['js'.$langcode] = array(
      '#type' => 'item',
      '#value' => '<a href="#" class="robotagger-js-select-all">' . $text . '</a>',
    );
  }
  $form['robotagger_nodetype_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Node type settings'),
    '#description' => t('Please note. You need to assign the content types for each vocabularies.'),
    '#collapsed' => TRUE,
  	'#collapsible' => TRUE,
  );

  foreach ($nodetypes as $mname => $uiname) {
    $w_key = 'robotagger_nodetype_' . $mname . '_wrapper';
    $form['robotagger_nodetype_wrapper'][$w_key] = array(
      '#type' => 'fieldset',
      '#title' => $uiname,
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
    );
    $s_key = 'robotagger_nodetype_' . $mname . '_settings';
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_use_global'] = array(
      '#type' => 'checkbox',
      '#title' => t('Use default settings'),
      '#description' => t('All made settings below are ignored and the global setting are used.'),
      '#default_value' => variable_get($s_key.'_use_global', 1),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_sendrequest'] = array(
      '#type' => 'select',
      '#title' => t('Send request'),
      '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
      '#default_value' => variable_get($s_key.'_sendrequest', 'auto'),
    );
    $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_confirmtags'] = array(
      '#type' => 'select',
      '#title' => t('Confirm returned tags'),
      '#options' => array('auto' => t('automatic'), 'manual' => t('manual')),
      '#default_value' => variable_get($s_key.'_confirmtags', 'auto'),
      '#description' => t('manual - Before the tags are saved, you can confirm or edit them. automatic - the module save the tags immediately'),
    );
    foreach ($languages as $langcode => $langname) {
      $options = _robotagger_get_vacabularies($langcode, 'db');
      if (empty($options)) {
        continue;
      }
      $values = variable_get($s_key.'_vocs_'.$langcode, array());
      $form['robotagger_nodetype_wrapper'][$w_key][$s_key.'_vocs_'.$langcode] = array(
        '#type' => 'checkboxes',
        '#title' => t('Vocabularies for the language @language', array('@language' => $langname)),
        '#options' => $options,
        '#default_value' => $values,
      );
    }
  }
  return system_settings_form($form);

}
