<?php

function _robotagger_process_node($node = FALSE, $type = '', $args = array()) {
  if ($type == 'nodeapi') {
    return _robotagger_process_node_nodeapi($node);
  }
  elseif ($type == 'form') {
    return _robotagger_process_node_form($args[0], $args[1]);
  }
}

function _robotagger_process_node_nodeapi($node) {
  if (empty($node->language)) {
    return;
  }
  $langcode = $node->language;
  $settings = _robotagger_get_settings($node->type, $langcode);
  if ($settings->sendrequest == 'disabled' || !$settings->isallow) {
    return;
  }
  if ($settings->sendrequest == 'manual' && $node->rtcheck_sendrequest == TRUE) {
    return;
  }
  $vids = array_filter($settings->vids);//remove disabled vids
  if (empty($vids)) {
    return;
  }
  $nodetypes_vids = array();
  $SQL = "SELECT v.* FROM {vocabulary} v
          INNER JOIN {vocabulary_node_types} n ON v.vid = n.vid
          WHERE n.type = '%s' AND v.language = '%s' AND v.vid IN(" . db_placeholders($vids, 'int') . ")";
  $query = db_query($SQL, array_merge(array($node->type, $langcode), $vids));
  while ($vocabulary = db_fetch_object($query)) {
    $nodetypes_vids[$vocabulary->vid] = $vocabulary->name;
  }

  if (!empty($nodetypes_vids)) {
    $data = robotagger_api_call_webservice($node->body, $nodetypes_vids, $langcode);
    if (empty($data)) {
      return;
    }
    watchdog('robotagger', 'Robotagger processing %content.', array('%content' => truncate_utf8($node->body, 150, TRUE, TRUE)));
    $terms = taxonomy_node_get_terms($node, $key = 'tid');
    foreach ($nodetypes_vids as $vid => $vocname) {
      if (isset($data[$vocname])) {
        $suggestions = array();
        foreach ($data[$vocname] as &$item) {
          if (($value = trim($item['value'])) && !empty($value)) {
            $suggestions[$value] = $item;
          }
        }
        if (!empty($suggestions)) {
          if ($settings->confirmtags == 'auto') {
            _robotagger_term_save($node, $vid, $suggestions, $terms);
          }
          elseif ($settings->confirmtags == 'manual') {
            _robotagger_term_draft_save($node, $vid, $suggestions);
          }
        }
      }
    }
  }

}

/**
 * Prepared the $node object for taxonomy_node_save().
 *
 * @see robotagger_node_page_form_submit();
 *
 * @param array $form
 * @param array $form_state
 */
function _robotagger_process_node_form($form, $form_state) {
  $node = $form['#node'];
  $all_terms = $suggestions = array();
  foreach ($node->taxonomy as $termobj) {
    $all_terms[$termobj->vid][$termobj->name] = $termobj->name;
  }
  foreach ($form_state['values']['vids'] as $vid => $tags) {
    foreach (array_keys(array_filter($tags)) as $name) {
      if (!isset($all_terms[$vid][$name])) {
        $all_terms[$vid][$name] = $name;
      }
      $suggestions[$vid][] = $name;
    }
  }
  $node->taxonomy = array();
  foreach ($all_terms as $vid => $terms) {
    $node->taxonomy['tags'][$vid] = drupal_implode_tags($terms);
  }
  if ($form_state['values']['action'] == 'confirm') {
    taxonomy_node_save($node, $node->taxonomy);
  }
  _robotagger_term_draft_delete($node, $suggestions);
}

function _robotagger_term_save($node, $vid, $suggestions, $terms) {
  $new_terms = $inserted = array();
  $names = array_keys($suggestions);
  $SQL = 'SELECT tid, name FROM {term_data} WHERE name IN(' . db_placeholders($names, 'varchar') . ') AND language = "%s" AND vid = %d';
  $query = db_query($SQL, array_merge($names, array($node->language, $vid)));
  $tids = array();
  while($result = db_fetch_object($query)) {
    $tids[$result->name] = $result->tid;
  }
  foreach ($suggestions as $suggestion) {
    if (isset($tids[$suggestion['value']])) {
      $tid =  $tids[$suggestion['value']];
      $new_terms[$tid] = $tid;
    }
    else{
      $edit = array(
        'vid' => $vid,
        'name' => $suggestion['value'],
      );
      $status = taxonomy_save_term($edit);
      if (isset($edit['tid']) && $edit['tid']) {
        $new_terms[$edit['tid']] = $edit['tid'];
      }
    }
  }
  if (!empty($new_terms)) {
    foreach ($new_terms as $tid) {
      if (!isset($inserted[$tid]) && !isset($terms[$tid])) {
        db_query('INSERT INTO {term_node} (nid, vid, tid) VALUES (%d, %d, %d)', $node->nid, $node->vid, $tid);
        $inserted[$tid] = TRUE;
      }
    }
  }

}

/**
 * Saves the terms in the robotagger db table, so that the user can confirm the tags.
 *
 * @param object $node
 * @param vid $vid
 * @param array $suggestions
 */
function _robotagger_term_draft_save($node, $vid, $suggestions) {
  if (isset($node->nid)) {
    $oldsuggestions = _robotagger_get_suggestions($node, $vid);
    $new_terms = array();
    foreach ($suggestions as $annotation => $suggestion) {
      if (!isset($oldsuggestions[$suggestion['value']])) {
        $new_terms[$suggestion['value']] = $suggestion;
      }
    }
    if (!empty($new_terms)) {
      $names = array_keys($new_terms);
      $SQL = 'SELECT tid, name FROM {term_data} WHERE name IN(' . db_placeholders($names, 'varchar') . ') AND language = "%s" AND vid = %d';
      $query = db_query($SQL, array_merge($names, array($node->language, $vid)));
      $tids = array();
      while ($result = db_fetch_object($query)) {
        $tids[$result->name] = $result->tid;
      }
      $new = FALSE;
      foreach ($new_terms as $new_term) {
        if (!isset($tids[$new_term['value']])) {
          db_query("INSERT INTO {robotagger} (nid, nvid, vid, annotation, occurences, threshold) VALUES( %d, %d, %d, '%s', %d, '%s')", $node->nid, $node->vid, $vid, $new_term['value'], $new_term['occurences'], $new_term['threshold']);
          $new = TRUE;
        }
      }
      if ($new) {
        $link = l(t('new tags'), 'node/' . $node->nid . '/robotagger');
        drupal_set_message(t('The RoboTagger-Webservice was requested. Please confirm the returned !new_tags', array('!new_tags' => $link)), 'status', FALSE);
      }
    }
  }
}

/**
 * Removes/Updates terms from the robotagger db table.
 *
 * @param object $node
 * @param array $suggestions
 */
function _robotagger_term_draft_delete($node, $suggestions) {
  if (!empty($suggestions) && !empty($node->nid)) {
    foreach ($suggestions as $vid => $names) {
     foreach ($names as $name) {
       db_query("DELETE FROM {robotagger} WHERE nid = %d AND nvid = %d AND vid = %d AND annotation = '%s'", $node->nid, $node->vid, $vid, $name);
     }
    }
  }
}

